"use strict";

require("core-js/modules/es7.symbol.async-iterator");

require("core-js/modules/es6.symbol");

require("core-js/modules/web.dom.iterable");

require("core-js/modules/es6.array.iterator");

require("core-js/modules/es6.string.iterator");

require("core-js/modules/es6.set");

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

var __importDefault = void 0 && (void 0).__importDefault || function (mod) {
  return mod && mod.__esModule ? mod : {
    "default": mod
  };
};

Object.defineProperty(exports, "__esModule", {
  value: true
});

var fast_deep_equal_1 = __importDefault(require("fast-deep-equal"));
/**
 * Functions to be used in {@link Array.prototype.filter} as a callback.
 *
 * @example
 * [1, 2, 2].filter(toBe.unique)
 */


var toBe;

(function (toBe) {
  /**
   * Function to be used in {@link Array.prototype.filter} as a callback.
   * Filters out items that are not present ({@code undefined} and {@code null}) in array
   *
   * @returns {boolean}
   */
  function present(value) {
    return value != undefined;
  }

  toBe.present = present;
  /**
   * Function to be used in {@link Array.prototype.filter} as a callback.
   * Filters out items that are empty
   *
   * @returns {boolean}
   */

  function notEmpty(value) {
    return value != undefined && (Array.isArray(value) ? value.length !== 0 : true) && (typeof value === 'string' ? value !== '' : true);
  }

  toBe.notEmpty = notEmpty;
  /**
   * Function to be used in {@link Array.prototype.filter} as a callback.
   * Filters out items, that are not equal to provided item in parameters.
   * Objects are compared to be deep equal.
   *
   * @returns {function}
   */

  function equal(valueToMatch) {
    return function _equal(value) {
      return fast_deep_equal_1.default(valueToMatch, value);
    };
  }

  toBe.equal = equal;
  /**
   * Function to be used in {@link Array.prototype.filter} as a callback.
   * Filters out items, that are equal to provided item in parameters.
   * Objects are compared to be deep equal.
   *
   * @returns {function}
   */

  function notEqual(valueToMatch) {
    return function _notEqual(value) {
      return !fast_deep_equal_1.default(valueToMatch, value);
    };
  }

  toBe.notEqual = notEqual;
  /**
   * Function to be used in {@link Array.prototype.filter} as a callback.
   * Determines uniqueness of an objects in array. Be aware that if value
   * is not primitive, deep object equality will be checked to determine
   * uniqueness.
   *
   * @returns () => {boolean}
   *
   * @example
   * [1, 2, 2].filter(toBe.unique())
   * // [1, 2]
   * [{a: 1}, {a: 1}, {a: 2}].filter(toBe.unique())
   * // [{a: 1}, {a: 2}]
   */

  function unique() {
    var set = new Set();
    return function _unique(value) {
      var check = value;

      if (_typeof(value) === 'object') {
        check = JSON.stringify(value);
      }

      if (!set.has(check)) {
        set.add(check);
        return true;
      }

      return false;
    };
  }

  toBe.unique = unique;

  function uniqueBy(toComparableProp) {
    var set = new Set();

    if (typeof toComparableProp === 'string') {
      return function _uniqueBy(value) {
        var check = value[toComparableProp];

        if (set.has(check)) {
          return false;
        } else {
          set.add(check);
          return true;
        }
      };
    } else if (typeof toComparableProp === 'function') {
      return function _uniqueBy(value) {
        var check = toComparableProp(value);

        if (set.has(check)) {
          return false;
        } else {
          set.add(check);
          return true;
        }
      };
    } else {
      // tslint:disable-next-line:max-line-length
      throw new Error("toBe.uniqueBy expected to have as a parameter string or function, instead got ".concat(_typeof(toComparableProp)));
    }
  }

  toBe.uniqueBy = uniqueBy;
  /**
   * Function to be used in {@link Array} filter function as a callback.
   * It will pass items from array, while predicate matches. When predicate
   * returns {@code false} none of the items will pass.
   *
   * @param {function} predicate callback function that returns boolean
   * @example
   * import {toBe} from 'declarative-js'
   * import takeWhile = toBe.takeWhile
   *
   * [
   *  { title: 'Predator', genre: 'sci-fi' },
   *  { title: 'Predator 2', genre: 'sci-fi'},
   *  { title: 'Tom & Jerry', genre: 'cartoon' },
   *  { title: 'Alien vs Predator', genre: 'sci-fi' },
   * ]
   *   .filter(takeWhile(film => film.genre === 'sci-fi'))
   * // =>
   * // [
   * //  { title: 'Predator', genre: 'sci-fi' },
   * //  { title: 'Predator 2', genre: 'sci-fi' }
   * // ]
   */

  function takeWhile(predicate) {
    var is = false;
    return function _takeWhile(value, index) {
      if (index === 0 || is) {
        is = predicate(value);
        return is;
      }

      return is;
    };
  }

  toBe.takeWhile = takeWhile;
})(toBe = exports.toBe || (exports.toBe = {}));