"use strict";

require("core-js/modules/es6.string.iterator");

require("core-js/modules/es6.array.from");

require("core-js/modules/es6.regexp.to-string");

require("core-js/modules/es7.symbol.async-iterator");

require("core-js/modules/es6.symbol");

require("core-js/modules/web.dom.iterable");

require("core-js/modules/es6.array.iterator");

require("core-js/modules/es6.object.keys");

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

var __importDefault = void 0 && (void 0).__importDefault || function (mod) {
  return mod && mod.__esModule ? mod : {
    "default": mod
  };
};

Object.defineProperty(exports, "__esModule", {
  value: true
});

var fast_deep_equal_1 = __importDefault(require("fast-deep-equal"));

var ImmutableBuilder_1 = require("../map/ImmutableBuilder");

var toObject_1 = require("../internal/toObject");

var toMap_1 = require("../internal/toMap");

var JMap_1 = require("../map/JMap");

var reducer_utils_1 = require("../internal/reducer.utils");
/**
 * Functions to be used in {@link Array.prototype.reduce} as a callback.
 *
 * @example
 * [
 *  { title: 'Predator', genre: 'scy-fy },
 *  { title: 'Predator 2', genre: 'scy-fy},
 *  { title: 'Alien vs Predator', genre: 'scy-fy },
 *  { title: 'Tom & Jerry', genre: 'cartoon },
 * ]
 *  .reduce(Reducer.groupBy('genre'), Reducer.Map())
 */


var Reducer;

(function (Reducer) {
  function Map(data) {
    return new JMap_1.JMap(data);
  }

  Reducer.Map = Map;

  function ImmutableMap() {
    return new ImmutableBuilder_1.ImmutableBuilder();
  }

  Reducer.ImmutableMap = ImmutableMap;

  function ImmutableObject() {
    var object = {};
    return Object.defineProperty(object, 'immutable', {
      value: true,
      enumerable: false
    });
  }

  Reducer.ImmutableObject = ImmutableObject;

  function groupBy(getKey) {
    var transformer = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function (x) {
      return x;
    };

    switch (_typeof(getKey)) {
      case 'string':
        {
          var key = getKey;
          return function (agr, value, index, array) {
            var derivedKey = value[key];

            if (typeof derivedKey === 'string') {
              var derivedValue = agr.get(derivedKey);

              if (derivedValue) {
                derivedValue.push(transformer(value));
              } else {
                agr.put(derivedKey, [transformer(value)]);
              }

              return reducer_utils_1.isLastElement(array, index) ? reducer_utils_1.finalizeMap(agr) : agr;
            } // tslint:disable-next-line:max-line-length


            throw new Error('Value of "' + key + '" in groupBy ' + ' must be string, instead get: ' + _typeof(value[key]));
          };
        }

      case 'function':
        {
          return function (agr, value, index, array) {
            var key = reducer_utils_1.valid(getKey(value));
            var extractedValue = agr.get(key);

            if (extractedValue !== void 0) {
              extractedValue.push(transformer(value));
            } else {
              agr.put(key, [transformer(value)]);
            }

            return reducer_utils_1.isLastElement(array, index) ? reducer_utils_1.finalizeMap(agr) : agr;
          };
        }

      default:
        // tslint:disable-next-line:max-line-length
        throw new Error("Reducer.groupBy function accepts as a paramter string or callback, instead got ".concat(_typeof(getKey)));
    }
  }

  Reducer.groupBy = groupBy;
  /**
   * Function to be used in {@link Array.prototype.reduce} as a callback
   * to make from 2d array simple array
   * As second parameter in reduce function need to pass <code>[]</code>
   * @param {T[]} agr              to collect in
   * @param {T[]} value            to concatenate with
   * @returns {T[]}                concatenated array
   * @example
   * [[1,2],[3,4]].reduce(flat, []) // [1,2,3,4]
   */

  Reducer.flat = function (agr, value) {
    if (Array.isArray(value)) {
      for (var index = 0; index < value.length; index++) {
        var element = value[index];
        agr[agr.length] = element;
      }
    } else {
      agr[agr.length] = value;
    }

    return agr;
  };

  function toMap(getKey, valueGetter) {
    var mapper = valueGetter === undefined ? toMap_1.toMapKeyMap(getKey) : toMap_1.toMapAndValue(getKey, valueGetter);
    return function _toMap(agr, value, index, array) {
      return mapper(agr, value, index, array);
    };
  }

  Reducer.toMap = toMap;

  function toObject(getKey, valueGetter, merge) {
    var onDuplicate = merge || reducer_utils_1.onDuplacateDefaultFunction;
    var reducer = valueGetter === undefined ? toObject_1.toObjectValueObject(getKey) : toObject_1.toObjectAndValue(getKey, valueGetter, onDuplicate);
    return function _toObject(agr, value, index, array) {
      return reducer(agr, value, index, array);
    };
  }

  Reducer.toObject = toObject;
  /**
   * Function to be used in {@link Array.prototype.reduce} as a callback.
   * Finds lowest value in array. Array must contain only numbers
   * @returns {number} lowest value in array.
   *
   * @example
   * [1, 2, 3].reduce(Reducer.min) // 1
   */
  // @ts-ignore

  function min(agr, value, index, array) {
    return reducer_utils_1.isLastElement(array, index) ? Math.min.apply(Math, _toConsumableArray(array)) : 0;
  }

  Reducer.min = min;
  /**
   * Function to be used in {@link Array.prototype.reduce} as a callback.
   * Finds highest value in array. Array must contain only numbers
   * @returns {number} highest value in array.
   *
   * @example
   * [1, 2, 3].reduce(Reducer.max) // 3
   */
  // @ts-ignore

  function max(agr, value, index, array) {
    return reducer_utils_1.isLastElement(array, index) ? Math.max.apply(Math, _toConsumableArray(array)) : 0;
  }

  Reducer.max = max;
  /**
   * Function to be used in {@link Array.prototype.reduce} as a callback.
   * Finds sum of values in array. Array must contain only numbers
   * @returns {number} sum of values in array.
   *
   * @example
   * [1, 2, 3].reduce(Reducer.sum) // 6
   */

  Reducer.sum = function (agr, value) {
    return agr + value;
  };
  /**
   * Object merging strategy used in {@link Reducer#toMergedObject}
   * @see toMergedObject
   */


  Reducer.MergeStrategy = {
    /**
     * Overrides value by duplicated key while merging objects
     */
    OVERRIDE: function OVERRIDE() {
      return true;
    },

    /**
     * Keys in objects must be unique
     */
    UNIQUE: function UNIQUE(aggregatorValue) {
      return aggregatorValue == null;
    },

    /**
     * Keys in objects may have duplicates, but values in these key must be equal
     */
    CHECKED: function CHECKED(aggregatorValue, currentValue) {
      return aggregatorValue == null || fast_deep_equal_1.default(aggregatorValue, currentValue);
    }
  };
  /**
   * Function to be used in {@link Array.prototype.reduce} as a callback.
   * Reduces array of objects to one object, There is three merge strategies
   * @see MergeStrategy
   * @param merge {@link MergeStrategy} = default is OVERRIDE
   */

  function toMergedObject() {
    var isMergable = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : Reducer.MergeStrategy.OVERRIDE;
    return function _toMergedObject(agr, value) {
      for (var _i = 0, _Object$keys = Object.keys(value); _i < _Object$keys.length; _i++) {
        var k = _Object$keys[_i];
        var valueFromAggr = agr[k];
        var valueFromObject = value[k];

        if (!isMergable(valueFromAggr, valueFromObject, k)) {
          // tslint:disable-next-line:max-line-length
          throw new Error("Failed to merge objects. Check the merging predicate (\"strategy\") and objects in an array with key: ".concat(k));
        }

        agr[k] = valueFromObject;
      }

      return agr;
    };
  }

  Reducer.toMergedObject = toMergedObject;
  /**
   * Function to be used in {@link Array.prototype.reduce} as a callback.
   * Collects two arrays into one array of tuples, two element array([x ,y]).
   * The length of zipped array will be length of shortest array.
   *
   * @param {Array} array array to zip with
   * @returns array with elements from two arrays as tuples
   * @example
   *
   * // array lengths are equal
   * let a1 = [1, 2, 3]
   * let a2 = ['x', 'y', 'z']
   * let zipped = a1.reduce(Reducer.zip(a2), [])
   * // [[1, 'x'], [2, 'y'], [3, 'z']]
   *
   * // origin array is longer
   * let b1 = [1, 2, 3, 4]
   * let b2 = ['x', 'y', 'z']
   * let zipped = b1.reduce(Reducer.zip(b2), [])
   * // [[1, 'x'], [2, 'y'], [3, 'z']]
   *
   * // zip array is longer
   * let c1 = [1, 2, 3]
   * let c2 = ['x', 'y', 'z', 'extra']
   * let zipped = c1.reduce(Reducer.zip(c2), [])
   * // [[1, 'x'], [2, 'y'], [3, 'z']]
   */

  function zip(array) {
    var isZipped = false;
    var secondArrLength = array.length;
    return function _zip(agr, value, index) {
      if (isZipped) {
        return agr;
      }

      var arrayValue = array[index];

      if (secondArrLength == index) {
        isZipped = true;
        return agr;
      }

      agr.push([value, arrayValue]);
      return agr;
    };
  }

  Reducer.zip = zip;
  /**
   * Function to be used in {@link Array.prototype.reduce} as a callback.
   * Collects all arrays to arrays of arrays, with elements
   * at being grouped with elements from other arrays by same index.
   * The length of zipped array will be length of shortest array.
   * Almost the same as {@link Reducer.zip}, except zipAll accepts
   * multiple array to zip with.
   *
   * @export
   * @param {...Array[]} arraysToZip
   * @returns function to use in Array.reduce
   * @see Reducer.zip
   * @example
   * import { Reducer } from 'declarative-js'
   * import zipAll = Reducer.zipAll'
   *
   * let numbers = [1, 2]
   * let chars = ['a', 'b']
   * let booleans = [true, false]
   * let result = numbers.reduce(zipAll(chars, booleans), [])
   * // [[1, 'a', true], [2, 'b', false]]
   *
   * let numbers1 = [1, 2, 3, 4, 5]
   * let chars1 = ['a', 'b']
   * let booleans1 = [true, false]
   * let result1 = numbers.reduce(zipAll(chars, booleans), [])
   * // [[1, 'a', true], [2, 'b', false]]
   */

  function zipAll() {
    for (var _len = arguments.length, arraysToZip = new Array(_len), _key = 0; _key < _len; _key++) {
      arraysToZip[_key] = arguments[_key];
    }

    var isZipped = false;
    var lengthOfArrays = arraysToZip.length;
    return function _zipAll(agr, currentValue, currentValueIndex) {
      if (isZipped) {
        return agr;
      }

      if (lengthOfArrays == currentValueIndex) {
        isZipped = true;
        return agr;
      }

      var zipee = [currentValue];

      for (var index = 0; index < arraysToZip.length; index++) {
        zipee.push(arraysToZip[index][currentValueIndex]);
      }

      agr.push(zipee);
      return agr;
    };
  }

  Reducer.zipAll = zipAll;
  /**
   * Function to be used in {@link Array.prototype.reduce} as a callback.
   * It does the opposite as {@link Reducer.zip} or {@link Reducer.zipAll}.
   * It collects from all zipped arrays one arrays, that was before zip.
   * Takes from each nested arrays and element and for each index will
   * collect to new array.
   * The length of and array will be the shortest length of arrays to unzip
   *
   * @export
   * @returns function to use in Array.reduce
   * @see Reducer.zip
   * @see Reducer.zipAll
   * @example
   *
   * import { Reducer } from 'declarative-js'
   * import zipAll = Reducer.zipAll
   * import unzip = Reducer.unzip
   *
   * let zipped = [[1, 'a', true], [2, 'b', false]]
   * zipped.reduce(unzip(), [])
   * // [
   * //   [1, 2],
   * //   ['a', 'b'],
   * //   [true, false]
   * // ]
   *
   * let zippedDifferentLength = [[1, 'a'], [2, 'b', false]]
   * zippedDifferentLength.reduce(unzip(), [])
   * // [
   * //   [1, 2],
   * //   ['a', 'b']
   * // ]
   *
   * let numbers = [1, 2]
   * let chars = ['a', 'b']
   * let booleans = [true, false]
   * let zipped = numbers.reduce(zipAll(chars, booleans), [])
   * zipped.reduce(unzip(), [])
   * // matches content
   * // [
   * //   numbers,
   * //   chars,
   * //   booleans
   * // ]
   */

  function unzip() {
    var zippersLength; // @ts-ignore

    return function _unzip(agr, value, index, arrays) {
      if (zippersLength == null) {
        zippersLength = arrays.map(function (arr) {
          return arr.length;
        }).reduce(min);
      }

      for (var valueArrayIndex = 0; valueArrayIndex < zippersLength; valueArrayIndex++) {
        var agrUnzipArray = agr[valueArrayIndex];

        if (agrUnzipArray) {
          agrUnzipArray.push(value[valueArrayIndex]);
        } else {
          agr[valueArrayIndex] = [value[valueArrayIndex]];
        }
      }

      return agr;
    };
  }

  Reducer.unzip = unzip;

  Reducer.Partition = function () {
    return [[], []];
  };

  function partitionBy(matches) {
    // tslint:disable-next-line
    var errorMessage = "Predicate for 'partitionBy' can be key of object, predicate function or partial object to match, instead got '".concat(matches, "'");
    var predicate;

    if (typeof matches === 'string') {
      predicate = function predicate(value) {
        return Boolean(value[matches]);
      };
    } else if (typeof matches === 'function') {
      predicate = function predicate(value) {
        return matches(value);
      };
    } else if (_typeof(matches) === 'object') {
      if (matches === null) {
        throw new Error(errorMessage);
      }

      predicate = function predicate(value) {
        return Object.keys(matches).every(function (key) {
          return value[key] === matches[key];
        });
      };
    } else {
      throw new Error(errorMessage);
    }

    return function _partitionByProp(agr, value) {
      if (predicate(value)) {
        agr[0].push(value);
      } else {
        agr[1].push(value);
      }

      return agr;
    };
  }

  Reducer.partitionBy = partitionBy;
})(Reducer = exports.Reducer || (exports.Reducer = {}));