"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var sort_1 = require("../internal/sort");
/**
 * Functions to be used in {@link Array.prototype.sort} as a callback.
 */


var Sort;

(function (Sort) {
  function ascendingBy() {
    var ascending = {
      true: 1,
      false: -1
    };
    var sort = typeof (arguments.length <= 0 ? undefined : arguments[0]) === 'string' ? sort_1.sortByKeyValues(ascending).apply(void 0, arguments) : sort_1.sortByGetters(ascending).apply(void 0, arguments);
    return function _ascendingBy(a, b) {
      return sort(a, b);
    };
  }

  Sort.ascendingBy = ascendingBy;

  function descendingBy() {
    for (var _len = arguments.length, getters = new Array(_len), _key = 0; _key < _len; _key++) {
      getters[_key] = arguments[_key];
    }

    return function _descendingBy(a, b) {
      var descending = {
        true: -1,
        false: 1
      };
      var sort = typeof getters[0] === 'string' ? sort_1.sortByKeyValues(descending).apply(void 0, getters) : sort_1.sortByGetters(descending).apply(void 0, getters);
      return sort(a, b);
    };
  }

  Sort.descendingBy = descendingBy;

  function by() {
    var sort = typeof (arguments.length <= 0 ? undefined : arguments[0]) === 'string' ? sort_1.sortByPropertyAndPriority(arguments.length <= 0 ? undefined : arguments[0], arguments.length <= 1 ? undefined : arguments[1]) : sort_1.sortByConditions.apply(sort_1, arguments);
    return function _by(a, b) {
      return sort(a, b);
    };
  }

  Sort.by = by;
  /**
   * Functions to be used in {@link Array.prototype.sort} as a callback.
   * Function that will sort items in array, by provided order.
   * It accepts as a parameter array of custom order rule.
   * Element, that are not present in order array will be at he the end of the sorted list.
   * @param order     array of custom order of items that are being sorted.
   * @returns comparator for Array.prototype.sort function.
   * @example
   * const testData =
   *   ['bar', 'medium', 'foo', 'low']
   * const result =
   *   testData.sort(orderedBy(['low', 'medium', 'high']))
   * // result => ['low', 'medium', 'bar', 'foo', ]
   */

  function orderedBy(order) {
    return function _orderedBy(a, b) {
      var condition = {
        toValue: function toValue(x) {
          return x;
        },
        order: order
      };
      return sort_1.sortByConditions.apply(sort_1, [condition])(a, b);
    };
  }

  Sort.orderedBy = orderedBy;
})(Sort = exports.Sort || (exports.Sort = {}));