"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

function comparatorValue(order, value) {
  var index = order.indexOf(value);
  return index === -1 ? order.length : index;
}
/**
 * @ignore
 * @internal
 */


function sortByConditions() {
  for (var _len = arguments.length, conditions = new Array(_len), _key = 0; _key < _len; _key++) {
    conditions[_key] = arguments[_key];
  }

  return function (a, b) {
    for (var index = 0; index < conditions.length; index++) {
      var _conditions$index = conditions[index],
          order = _conditions$index.order,
          toValue = _conditions$index.toValue;
      var aa = comparatorValue(order, toValue(a));
      var bb = comparatorValue(order, toValue(b));

      if (aa !== bb) {
        return aa > bb ? 1 : -1;
      }
    }

    return 0;
  };
}

exports.sortByConditions = sortByConditions;
/**
 * @ignore
 */

function sortByPropertyAndPriority(key, values) {
  return function (a, b) {
    var aa = comparatorValue(values, a[key]);
    var bb = comparatorValue(values, b[key]);

    if (aa !== bb) {
      return aa > bb ? 1 : -1;
    }

    return 0;
  };
}

exports.sortByPropertyAndPriority = sortByPropertyAndPriority;
/**
 * @ignore
 */

function sortByGetters(numbers) {
  return function () {
    for (var _len2 = arguments.length, mappersArr = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      mappersArr[_key2] = arguments[_key2];
    }

    return function (a, b) {
      for (var index = 0; index < mappersArr.length; index++) {
        var mapper = mappersArr[index];
        var aa = mapper(a);
        var bb = mapper(b);

        if (aa !== bb) {
          return aa > bb ? numbers.true : numbers.false;
        }
      }

      return 0;
    };
  };
}

exports.sortByGetters = sortByGetters;
/**
 * @ignore
 */

function sortByKeyValues(numbers) {
  return function () {
    for (var _len3 = arguments.length, keys = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      keys[_key3] = arguments[_key3];
    }

    return function (a, b) {
      for (var index = 0; index < keys.length; index++) {
        var key = keys[index];
        var aa = a[key];
        var bb = b[key];

        if (aa !== bb) {
          return aa > bb ? numbers.true : numbers.false;
        }
      }

      return 0;
    };
  };
}

exports.sortByKeyValues = sortByKeyValues;